"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const bufwrapper_1 = require("@minecraft-js/bufwrapper");
const node_events_1 = require("node:events");
const protocol = require(".");
const constants_1 = require("../constants");
const BufWrapperLunarPlugin = require("./BufWrapperPlugin");
const TransferPacket_1 = require("./client/TransferPacket");
const events = Object.keys(protocol);
const packets = Object.values(protocol);
/**
 * Instantiate a new PacketHandler class.
 * This class extends EventEmitter and for
 * each successfuly handled packet the event
 * emitter will emit an event. If a `CooldownPacket`
 * is received, an event with name `CooldownPacket`
 * is emitted.
 *
 * @example
 * ```javascript
 * const packetReader = new LunarClientPacketReader();
 *
 * // Call PacketReader#read with
 * // the buffer everytime you receive
 * // a packet
 *
 * packetReader.on('CooldownPacket', ({ data }) => {
 *   // Do something with data
 * });
 * ```
 */
class LunarClientPacketReader extends node_events_1.EventEmitter {
    /**
     * Read a packet, if everything is successful
     * an event will be emited with the content of
     * the packet
     * @param buffer The raw packet to read
     * @param channel The plugin channel the packet comes from. Defauts to `LunarClientPacketReader.NEW`
     */
    read(buffer, channel) {
        channel = channel ?? constants_1.LunarClientPluginChannel.NEW;
        const buf = new bufwrapper_1.BufWrapper(buffer, {
            plugins: { lunar: BufWrapperLunarPlugin },
        });
        if (channel === constants_1.LunarClientPluginChannel.TRANSFER) {
            const packet = new TransferPacket_1.TransferPacket(buf);
            packet.read();
            return void this.emit('TransferPacket', packet);
        }
        const id = buf.readVarInt();
        const Packet = packets.find((p) => p.id === id);
        if (!Packet)
            throw new Error(`Unknown packet (id=${id})`);
        const packet = new Packet(buf);
        packet.read();
        const event = events.find((key) => packet[key] === Packet);
        // @ts-ignore
        this.emit(event, packet);
    }
}
exports.default = LunarClientPacketReader;
